package anywheresoftware.b4a.objects;

import android.content.ComponentName;
import android.content.Intent;
import android.content.SharedPreferences;
import android.net.Uri;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceGroup;
import android.preference.PreferenceScreen;
import android.preference.RingtonePreference;
import android.text.method.PasswordTransformationMethod;
import android.widget.EditText;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.keywords.Common;
import anywheresoftware.b4a.objects.collections.List;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashSet;

@BA.Version(1.03F)
@BA.Author("Erel Uziel / Markus Stipp / Francesco Iafulli")
public class preferenceactivity extends PreferenceActivity
{
  @SuppressWarnings("deprecation")
public void onCreate(Bundle savedInstanceState)
  {
    super.onCreate(savedInstanceState);
    Intent in = getIntent();
    if ((in == null) || (!in.hasExtra("preferences"))) {
      Common.Log("Intent missing root node.");
      return;
    }

    PreferenceScreenWrapper p = (PreferenceScreenWrapper)in.getSerializableExtra("preferences");
    PreferenceScreen ps = (PreferenceScreen)p.createPreference(this);
    setPreferenceScreen(ps);

    setDependency(ps, p.childs);
  }

  private void setDependency(PreferenceScreen ps, ArrayList<B4APreference> childs) {
    for (B4APreference b : childs)
      if (b.key != null)
        if ((b instanceof PreferenceCategoryWrapper)) {
          setDependency(ps, ((PreferenceCategoryWrapper)b).childs);
        }
        else if ((b instanceof PreferenceScreenWrapper)) {
          if ((b.dependency.length() > 0) && (ps.findPreference(b.dependency) == null))
            Common.Log("Dependency key does not exist: " + b.dependency);
          else {
            ps.findPreference(b.key).setDependency(b.dependency);
          }
          setDependency(ps, ((PreferenceScreenWrapper)b).childs);
        }
        else if ((b.dependency.length() > 0) && (ps.findPreference(b.dependency) == null)) {
          Common.Log("Dependency key does not exist: " + b.dependency);
        } else {
          ps.findPreference(b.key).setDependency(b.dependency);
        }
  }

  @SuppressWarnings("serial")
static abstract class B4APreference
    implements Serializable
  {
    String key;
    Serializable defaultValue;
    String dependency;
    String title;
    String summary;
    int InputType;
    boolean Password;
    boolean SelectAllOnFocus;
    String intentAction;
    String intentDataString;
    String intentPackage;
    String intentClass;
    protected static int dummyKey;

    public B4APreference(String key, String title, String summary, Serializable defaultValue, String dependency)
    {
      this.key = key;
      this.defaultValue = defaultValue;
      this.summary = summary;
      this.title = title;
      this.dependency = dependency;
    }
    public B4APreference(String key, String title, String summary, Serializable defaultValue, boolean SelectAllOnFocus, String dependency)
    {
      this.key = key;
      this.defaultValue = defaultValue;
      this.summary = summary;
      this.title = title;
      this.dependency = dependency;
      this.SelectAllOnFocus = SelectAllOnFocus;
    }
    public B4APreference(String key, String title, String summary, Serializable defaultValue, int InputType, boolean Password, boolean SelectAllOnFocus, String dependency)
    {
      this.key = key;
      this.defaultValue = defaultValue;
      this.summary = summary;
      this.title = title;
      this.dependency = dependency;
      this.InputType = InputType;
      this.Password = Password;
      this.SelectAllOnFocus = SelectAllOnFocus;
    }
    public B4APreference(String title, String summary, String dependency, String intentAction, ComponentName intentComponent, Uri intentData) {
      this.key = ("_B4APreferenceDummy" + Integer.toString(++dummyKey));
      this.summary = summary;
      this.title = title;
      this.dependency = dependency;
      this.intentAction = intentAction;
      if (intentData != null)
        this.intentDataString = intentData.toString();
      else {
        this.intentDataString = "";
      }
      if (intentComponent != null) {
        this.intentPackage = intentComponent.getPackageName();
        this.intentClass = intentComponent.getClassName();
      }
      else {
        this.intentPackage = "";
        this.intentClass = "";
      }
    }
    abstract Preference createPreference(PreferenceActivity paramPreferenceActivity);

    protected void handleDefaults(Preference p) { p.setKey(this.key);
      p.setDefaultValue(this.defaultValue);
      p.setTitle(this.title);
      p.setSummary(this.summary);
    }
  }

  @SuppressWarnings("serial")
@BA.ShortName("AHPreferenceCategory")
  public static class PreferenceCategoryWrapper extends preferenceactivity.PreferenceScreenWrapper
  {
    PreferenceGroup parent;

    public PreferenceCategoryWrapper()
    {
      super();
    }

    @SuppressWarnings({ "unchecked", "rawtypes" })
	public void Initialize(String Title)
    {
      this.childs = new ArrayList();
      this.title = Title;
    }

    Preference createPreference(PreferenceActivity c) {
      PreferenceCategory pc = new PreferenceCategory(c);
      handleDefaults(pc);
      if (this.parent == null) {
        Common.Log("Error: PreferenceCategories cannot be nested!");
      }
      this.parent.addPreference(pc);
      for (preferenceactivity.B4APreference b : this.childs) {
        pc.addPreference(b.createPreference(c));
      }
      return pc;
    }

    @BA.Hide
    public Intent CreateIntent() {
      return null;
    }
  }

  @BA.ShortName("AHPreferenceManager")
  public static class PreferenceManager
  {
    private SharedPreferences sp = android.preference.PreferenceManager.getDefaultSharedPreferences(BA.applicationContext);
    @SuppressWarnings({ "unchecked", "rawtypes" })
	private HashSet<String> updatedKeys = new HashSet();
    private final SharedPreferences.OnSharedPreferenceChangeListener listener;

    public PreferenceManager()
    {
      this.listener = new SharedPreferences.OnSharedPreferenceChangeListener()
      {
        public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key)
        {
          preferenceactivity.PreferenceManager.this.updatedKeys.add(key);
        }
      };
      this.sp.registerOnSharedPreferenceChangeListener(this.listener);
    }

    public static void LIBRARY_DOC()
    {
    }

    public List GetUpdatedKeys()
    {
      List l = new List();
      l.Initialize();
      for (String s : this.updatedKeys) {
        l.Add(s);
      }
      this.updatedKeys.clear();
      return l;
    }

    public String GetString(String Key)
    {
      return this.sp.getString(Key, "");
    }

    public boolean GetBoolean(String Key)
    {
      return this.sp.getBoolean(Key, false);
    }

    @SuppressWarnings("rawtypes")
	public anywheresoftware.b4a.objects.collections.Map GetAll()
    {
      anywheresoftware.b4a.objects.collections.Map m = new anywheresoftware.b4a.objects.collections.Map();
      m.Initialize();
      for (java.util.Map.Entry e : this.sp.getAll().entrySet()) {
        m.Put(e.getKey(), e.getValue());
      }
      return m;
    }

    public void SetString(String Key, String Value)
    {
      SharedPreferences.Editor e = this.sp.edit();
      e.putString(Key, Value);
      e.commit();
    }

    public void SetBoolean(String Key, boolean Value)
    {
      SharedPreferences.Editor e = this.sp.edit();
      e.putBoolean(Key, Value);
      e.commit();
    }

    public void ClearAll()
    {
      SharedPreferences.Editor e = this.sp.edit();
      e.clear();
      e.commit();
    }
  }

  @SuppressWarnings("serial")
@BA.ShortName("AHPreferenceScreen")
  public static class PreferenceScreenWrapper extends preferenceactivity.B4APreference
  {
    protected ArrayList<preferenceactivity.B4APreference> childs;
    protected static int randomKey;
    public int RT_RINGTONE = 1;
    public int RT_NOTIFICATION = 2;
    public int RT_ALARM = 4;

    public PreferenceScreenWrapper()
    {
      this("PreferenceScreen" + Integer.toString(++randomKey));
    }
    protected PreferenceScreenWrapper(String key) {
      super(null, null, null, null, null);
    }

    @SuppressWarnings({ "rawtypes", "unchecked" })
	public void Initialize(String Title, String Summary)
    {
      this.childs = new ArrayList();
      this.title = Title;
      this.summary = Summary;
    }

    public Intent CreateIntent()
    {
      Intent in = new Intent();
      in.setClass(BA.applicationContext, preferenceactivity.class);
      in.putExtra("preferences", this);
      return in;
    }

    @SuppressWarnings("deprecation")
	Preference createPreference(PreferenceActivity c) {
      PreferenceScreen ps = c.getPreferenceManager().createPreferenceScreen(c);
      handleDefaults(ps);
      for (preferenceactivity.B4APreference b : this.childs) {
        if ((b instanceof preferenceactivity.PreferenceCategoryWrapper)) {
          ((preferenceactivity.PreferenceCategoryWrapper)b).parent = ps;
          b.createPreference(c);
        }
        else {
          ps.addPreference(b.createPreference(c));
        }
      }
      return ps;
    }

    public void AddCheckBox(String Key, String Title, final String SummaryOn, final String SummaryOff, boolean DefaultValue, String Dependency)
    {
      this.childs.add(new preferenceactivity.B4APreference(Key, Title, SummaryOn, Boolean.valueOf(DefaultValue), Dependency)
      {
        Preference createPreference(PreferenceActivity c) {
          CheckBoxPreference cbp = new CheckBoxPreference(c);
          handleDefaults(cbp);
          if (SummaryOff.length() > 0) {
            cbp.setSummaryOn(SummaryOn);
            cbp.setSummaryOff(SummaryOff);
          }
          return cbp;
        }
      });
    }

    public void AddList(String Key, String Title, String Summary, String DefaultValue, String Dependency, List Values)
    {
      final CharSequence[] values = new String[Values.getSize()];
      for (int i = 0; i < values.length; i++) {
        values[i] = String.valueOf(Values.Get(i));
      }
      this.childs.add(new preferenceactivity.B4APreference(Key, Title, Summary, DefaultValue, Dependency)
      {
        Preference createPreference(PreferenceActivity c) {
          ListPreference list = new ListPreference(c);
          handleDefaults(list);
          list.setDialogTitle(this.title);

          list.setEntries(values);
          list.setEntryValues(values);
          return list;
        }
      });
    }

    public void AddList2(String Key, String Title, String Summary, String DefaultValue, String Dependency, anywheresoftware.b4a.objects.collections.Map Values)
    {
      final CharSequence[] values = new String[Values.getSize()];
      final CharSequence[] keys = new String[Values.getSize()];
      for (int i = 0; i < values.length; i++) {
        values[i] = String.valueOf(Values.GetValueAt(i));
        keys[i] = String.valueOf(Values.GetKeyAt(i));
      }
      this.childs.add(new preferenceactivity.B4APreference(Key, Title, Summary, DefaultValue, Dependency)
      {
        Preference createPreference(PreferenceActivity c) {
          ListPreference list = new ListPreference(c);
          handleDefaults(list);
          list.setDialogTitle(this.title);

          list.setEntries(values);
          list.setEntryValues(keys);
          return list;
        }
      });
    }

    public void AddRingtone(String Key, String Title, String Summary, String DefaultValue, String Dependency, final int RingToneType)
    {
      this.childs.add(new preferenceactivity.B4APreference(Key, Title, Summary, DefaultValue, Dependency)
      {
        Preference createPreference(PreferenceActivity c) {
          RingtonePreference ring = new RingtonePreference(c);
          handleDefaults(ring);
          ring.setRingtoneType(RingToneType);
          return ring;
        }
      });
    }

    public void AddEditText(String Key, String Title, String Summary, String DefaultValue, boolean SelectAllOnFocus, String Dependency)
    {
      this.childs.add(new preferenceactivity.B4APreference(Key, Title, Summary, DefaultValue, SelectAllOnFocus, Dependency)
      {
        Preference createPreference(PreferenceActivity c) {
          EditTextPreference e = new EditTextPreference(c);
          e.setDialogTitle(this.title);
          EditText et = e.getEditText();
          et.setSelectAllOnFocus(SelectAllOnFocus);
          handleDefaults(e);
          return e;
        }
      });
    }

    public void AddEditText2(String Key, String Title, String Summary, String DefaultValue, int InputType, boolean Password, boolean SelectAllOnFocus, String Dependency)
    {
      this.childs.add(new preferenceactivity.B4APreference(Key, Title, Summary, DefaultValue, InputType, Password, SelectAllOnFocus, Dependency)
      {
        Preference createPreference(PreferenceActivity c) {
          EditTextPreference e = new EditTextPreference(c);
          e.setDialogTitle(this.title);
          EditText et = e.getEditText();
          et.setInputType(InputType);
          et.setSelectAllOnFocus(SelectAllOnFocus);
          if (Password) et.setTransformationMethod(new PasswordTransformationMethod());
          handleDefaults(e);
          return e;
        }
      });
    }

    public void AddPassword(String Key, String Title, String Summary, String DefaultValue, boolean SelectAllOnFocus, String Dependency)
    {
      this.childs.add(new preferenceactivity.B4APreference(Key, Title, Summary, DefaultValue, SelectAllOnFocus, Dependency)
      {
        Preference createPreference(PreferenceActivity c) {
          EditTextPreference e = new EditTextPreference(c);
          e.setDialogTitle(this.title);
          EditText et = e.getEditText();
          et.setSelectAllOnFocus(SelectAllOnFocus);
          et.setTransformationMethod(new PasswordTransformationMethod());
          handleDefaults(e);
          return e;
        }
      });
    }

    public void AddPreferenceScreen(PreferenceScreenWrapper PreferenceScreen, String Dependency)
    {
      PreferenceScreen.dependency = Dependency;
      this.childs.add(PreferenceScreen);
    }

    public void AddIntent(String Title, String Summary, Intent Intent, String Dependency)
    {
      this.childs.add(new preferenceactivity.B4APreference(Title, Summary, Dependency, Intent.getAction(), Intent.getComponent(), Intent.getData())
      {
        @SuppressWarnings("deprecation")
		Preference createPreference(PreferenceActivity c) {
          PreferenceScreen ps = c.getPreferenceManager().createPreferenceScreen(c);
          handleDefaults(ps);

          Intent in = new Intent(this.intentAction);
          if (((this.intentPackage.length() > 0 ? 1 : 0) & (this.intentClass.length() > 0 ? 1 : 0)) != 0)
            in.setClassName(this.intentPackage, this.intentClass);
          if (this.intentDataString.length() > 0) {
            Uri uri = Uri.parse(this.intentDataString);
            in.setData(uri);
          }
          ps.setIntent(in);
          return ps;
        }
      });
    }

    public void AddPreferenceCategory(preferenceactivity.PreferenceCategoryWrapper PreferenceCategory)
    {
      this.childs.add(PreferenceCategory);
    }
  }
}